

pub trait CharacterSheet : Default {
    /// Character sheet inputs (stuff that are not calculated from different items), such as Name, Age, Strength
    fn inputs(&self) -> Vec<(String, EntryType)>;
    /// All fields in the character sheet
    fn fields(&self) -> Vec<(String, EntryType)>;
    /// Character sheet to display, ordered. `None`s can be used to convey a seperator (of any sort)
    fn display(&self) -> Vec<Option<(String, EntryType)>>;
    /// Gets a character sheet entry value
    fn get(&self, entry: &str) -> Option<EntryType>;
    /// Sets a cahrater sheet entry value
    fn set(&mut self, entry: &str, value: EntryType);
}

#[derive(Debug)]
pub enum EntryType {
    Number(i32),
    Text(String),
    Bool(bool),
}
impl Default for EntryType {
    fn default() -> Self {
        EntryType::Number(0)
    }
}
impl EntryType {
    pub fn as_num(&self) -> i32 {
        if let EntryType::Number(n) = *self {
            n
        }
        else {
            0
        }
    }
    pub fn as_text(&self) -> String {
        if let EntryType::Text(t) = self {
            t.clone()
        }
        else {
            String::new()
        }
    }
    pub fn as_bool(&self) -> bool {
        if let EntryType::Bool(b) = *self {
            b
        }
        else {
            false
        }
    }
}
impl std::fmt::Display for EntryType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match(self) {
            EntryType::Number(n) => write!(f, "{}", n),
            EntryType::Text(t) => write!(f, "{}", t),
            EntryType::Bool(b) => write!(f, "{}", b),
        }
    }
}