use axum::{
	extract::ws, response, routing, Router
};
use open_tavern::{game::character_sheet::{CharacterSheet, EntryType}, pathfinder2r_impl};

#[tokio::main]
async fn main() {
	let mut pf = pathfinder2r_impl::Pathfinder2rCharacterSheet::default();
	pf.set("Dexterity", EntryType::Number(10));
	pf.set("Name", EntryType::Text("AAAA".to_string()));
	pf.set("Items", EntryType::Text("Short Sword, Dragonleather Helmet".to_string()));

	let d = pf.display();
	for e in d {
		if let  Some((s, v)) = e {
			println!("{}: {}", s, v);
		}
		else {
			println!();
		}
	}
	println!();

		//    let app = Router::new()
		//    		.route("/", routing::get(root))
		//    		.route("/socket.js", routing::get(socket))
		//    		.route("/ws", routing::get(ws_handler))
		//    		;

		// 	let listener = tokio::net::TcpListener::bind("0.0.0.0:3001").await.unwrap();
		// 	axum::serve(listener, app).await.unwrap();
}

async fn ws_handler(ws: ws::WebSocketUpgrade) -> impl axum::response::IntoResponse {
	ws.on_upgrade(handle_socket)
}

async fn handle_socket(mut socket: ws::WebSocket) {
	if socket.send(ws::Message::Text("This is my test message!".to_string())).await.is_ok() {
		println!("pinged!");
	}
	let mut logged_in = false;

	loop {
		if let Some(msg) = socket.recv().await {
			if let Ok(msg) = msg {
				match msg {
					ws::Message::Text(t) =>  {
						println!("Got message: {}", t);
						if !logged_in && t.starts_with("login") {
							let mut split = t.splitn(2, ' ');
							split.next(); // the login part
							let user = split.next().unwrap();
							if open_tavern::user::User::default_admin().login(user) {
								socket.send(ws::Message::Text("ok".to_string())).await.unwrap();
								logged_in = true;
							}
							else {
								socket.send(ws::Message::Text("bad".to_string())).await.unwrap();
							}
						}
					}
					ws::Message::Binary(b) => println!("got bytes: {:?}", b),
					ws::Message::Ping(_) => todo!(),
					ws::Message::Pong(_) => todo!(),
					ws::Message::Close(_) => break,
				}
			}
		}
		else {
			break;
		}
	}
	println!("Done with so-cat");
}

async fn root() -> axum::response::Html<&'static str> {
	response::Html(include_str!("../assets/web/index.html"))
}

async fn socket() -> &'static str {
	include_str!("../assets/web/socket.js")
}

